package HTDig::Search;
use strict;
use URI::Escape;
use XML::Parser;
use Carp;

# This package searches based on a configuration file.
sub Search
{
	# These are the parameters we use - make sure you set this up to accept the parameters YOU use.
	my ($database, $words) = @_;
				
	$_ = uri_escape($_) foreach ($words);
	my $query = qq["words=$words"];

	# This is just for security...
	local @ENV;
	delete @ENV{qw[PATH IFS CDPATH ENV BASH_ENV]};

	# This is where htsearch gets called.
	my $xml = qx[/opt/htdig/bin/htsearch -c /opt/htdig/conf/$database.conf $query | tail +3]; 

	# Parse the XML
	my $p = new XML::Parser(Style=>'Tree');
	my $t;
	$t = $p->parse($xml);
	
	# start parsing the tree!
	my ($tag, $attr, $d) = Tag($t);
	die "\$tag is not 'SEARCH_RESULTS - it is '$tag'" unless ($tag eq 'SEARCH_RESULTS');
	return SearchResultsTag($d);
}

# This will take an array ref and get the tag, attributes, and data from it.
# It will destroy the array ref as it extracts needed info.
sub Tag
{
	my ($in) = @_;
	unless (ref $in eq 'ARRAY') {
		croak "Called 'Tag' on something that is not an ARRAY ref.";
	}
	return ('0', {}, '') unless (scalar @$in > 1);

	my $tag = shift @$in;
	my ($d, $attr);
	if ($tag) {
		$d = shift @$in;
		$attr = shift @$d;
	} else {
		$attr = {};
		$d = shift @$in;
	}
	return ($tag, $attr, $d);
}

# This will take data in tree format and crunch it into XML. 
sub Crunch
{
	my ($in) = @_;
	# Eat $in until it collapses
	croak "Called 'Crunch on something that is not an ARRAY ref." unless (ref $in eq 'ARRAY');
	my $crunch = '';
	while (scalar @$in > 0) {
		my ($tag, $attr, $d) = Tag($in);
		if ($tag) {
			if (scalar keys %$attr > 0) {
				$crunch .= "<$tag " . (join ' ', map {qq[$_="$attr->{$_}"]} keys %$attr) . ">";
			} else {
				$crunch .= "<$tag>";
			}
			$crunch .= Crunch($d);
			$crunch .= "</$tag>";
		} else {
			$crunch .= $d;
		}
	}
	$crunch =~ s/^\s+//sg;
	$crunch =~ s/\s+&//sg;
	return $crunch;
}

# This processes the SEARCH_RESULTS tag.
sub SearchResultsTag
{
	my ($in) = @_;
	my @matches = ();
	my %res = (MATCH_ARRAY=>\@matches);
	while (scalar @$in > 0)
	{
		my ($tag, $attr, $d) = Tag($in);
		if ($tag eq 'MATCH') {
			push @matches, MatchTag($d);
		} elsif ($tag) {
			# Crunch the data of this tag 
			my $crunch = Crunch($d);
			$res{$tag} = $crunch;
		} else {
			# ignore!
		}
	}
	sort {$a->{CURRENT} <=> $b->{CURRENT}} @{$res{MATCH_ARRAY}};
	return \%res;		
}

# This will process the MATCH tag.
sub MatchTag
{
	my ($in) = @_;
	my %res;
	while (scalar @$in > 0)
	{
		my ($tag, $attr, $d) = Tag($in);
		if ($tag) {
			my $crunch = Crunch($d);
			$res{$tag} = $crunch;
		} else {
			# ignore!
		}
	}
	return \%res;		
}


1;

